#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_PAGES 16

int main(int argc, char *argv[])
{
    char *inname=argv[1];
    char *outname=argv[2];
    FILE *infp,*outfp;
    int i;

    int page_num=0;
    int page_offs[MAX_PAGES];

    if (argc < 3) {
	fprintf(stderr,"too few arguments\n");
	fprintf(stderr,"usage: fixhelp <infile> <outfile>\n");
	exit(1);
    }
  
    infp=fopen(inname,"rb");
    if (!infp) {
	fprintf(stderr,"couldn't open infile\n");
	exit(1);
    }

    fseek(infp, 0, SEEK_END);
    int size=ftell(infp);
    fseek(infp, 0, SEEK_SET);
    char *buf=malloc(size);
    fread(buf,size, 1, infp);
    fclose(infp);

    /* first page at 0 */
    page_offs[page_num]=0;
    page_num++;

    /* scan for pages */
    char vstr[]="@@@\r";
    int vlen=strlen(vstr);
    int j=0;
    for (i=0; i < size; ++i) {
	if (vstr[j]!=buf[i]) {
	    j=0;
	    continue;
	}
	++j;
	if (j==vlen) {
	    j=0;
	    int pos=i+1;
	    int spos=i-(vlen-1);
	    printf("found 0x%04x, 0x%04x\n",spos, pos);
	    /* fixup */
	    if (spos-1 > 0 && buf[spos-1] == '\r')
		spos--;

	    buf[spos]=0;
	    spos++;
	    buf[spos]=0x9b;
	    spos++;

	    memmove(buf+spos,buf+pos,size-pos);
	    size-=pos-spos;

	    page_offs[page_num]=spos-1;
	    ++page_num;
	    
	}

    }
    /* truncate end */
    for (i=size-1; buf[i]==0x0d || buf[i]==0x00; --i, --size);
    printf("end 0x%04x\n",size);
    if (size-page_offs[page_num-1]==1) {
	size=page_offs[page_num-1];
	page_num--;
    }
    printf("%d pages\n", page_num);
    for (i=0; i < page_num; ++i) {
	printf("  page %d at 0x%04x\n",i+1, page_offs[i]);
    }

    outfp=fopen(outname,"wb");
    if (!outfp) {
	fprintf(stderr,"couldn't open outfile\n");
	exit(1);
    }

    int offs=1+page_num*2;
    fputc(page_num,outfp);
    for (i=0; i<page_num; i++) {
	int lsb=(page_offs[i]+offs) & 0xff;
	int msb=(page_offs[i]+offs) >> 8;
	fputc(lsb,outfp);
	fputc(msb,outfp);
    }
    fwrite(buf,size,1,outfp);

    fclose(outfp);

    free(buf);
    exit(0);
};

/* eof */
